@@ -1,7 +1,7 @@
 /* tsprintf.c -- test file for mpfr_sprintf, mpfr_vsprintf, mpfr_snprintf,
    and mpfr_vsnprintf
 
-Copyright 2007-2017 Free Software Foundation, Inc.
+Copyright 2007-2018 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -88,14 +88,11 @@ check_sprintf (const char *expected, const char *fmt, mpfr_srcptr x)
     }
   if (n0 != n1)
     {
-      char format[1024];
       printf ("Error in mpfr_snprintf (s, %d, \"%s\", x) return value\n",
               randsize, fmt);
-      printf ("expected: %d\ngot:      %d\n", n0, n1);
-      strncpy (format, "x='", 1024);
-      strncpy (format + 3, fmt, 1021);
-      strncpy (format + 3 + strlen (fmt), "'\n", 1021 - strlen (fmt));
-      mpfr_printf (format, x);
+      printf ("expected: %d\ngot:      %d\nx='", n0, n1);
+      mpfr_printf (fmt, x);
+      printf ("'\n");
       exit (1);
     }
   if ((randsize > 1 && strncmp (expected, buffer, randsize - 1) != 0)
